/*
 * $Id: unistd.h,v 1.10 2000/10/27 04:05:31 k-abe Exp $
 *
 * This file is part of PTL, Portable Thread Library.
 *
 * Copyright (C) 1997-2000 Kota Abe
 *
 */

#ifndef __PTL_UNISTD_H__
#define __PTL_UNISTD_H__

#include <sys/pthread-init.h>
#include <sys/pthread-cdecl.h>

/* ---------------------------------------------------------------- */
#ifdef __PTL_RENAMED__
#define __PTL_ALREADY_RENAMED_UNISTD_H
#else
#include <sys/pthread-before.h>
#endif

#ifdef HAVE_UNISTD_H
#ifdef __GNUC__
#include <00root-gcc/unistd.h>
#else
#include <00root-system/unistd.h>
#endif
#endif

#include <sys/pthread-after.h>
/* ---------------------------------------------------------------- */

/* struct sockaddr, struct msghdr */
#include <sys/socket.h>

/* struct iovec */
#include <sys/uio.h>



/****************************************************************
 *
 *	2.9.3  Compile-Time Symbolic Constants for Portability
 *             Specifications
 *
 ****************************************************************/

/* supports the Threads option */
#undef _POSIX_THREADS
#define	_POSIX_THREADS

/* not suports the Thread Stack Address Attribute option */
#undef _POSIX_THREAD_ATTR_STACKADDR

/* supports the Thread Stack Address Attribute option */
#undef _POSIX_THREAD_ATTR_STACKSIZE
#define _POSIX_THREAD_ATTR_STACKSIZE

/* supports the Thread Execution Scheduling option */
#undef _POSIX_THREAD_PRIORITY_SCHEDULING
#define _POSIX_THREAD_PRIORITY_SCHEDULING

/* supports the Priority Inheritance option */
#undef _POSIX_THREAD_PRIO_INHERIT
#define _POSIX_THREAD_PRIO_INHERIT

/* not suports the Priority Protection option */
#undef _POSIX_THREAD_PRIO_PROTECT

/* not supports the Process-Shared Synchronization option */
#undef _POSIX_THREAD_PROCESS_SHARED

/* supports the Thread-Safe Functions option */
#undef _POSIX_THREAD_SAFE_FUNCTIONS
#define _POSIX_THREAD_SAFE_FUNCTIONS

__PTL_BEGIN_DECLS

/****************************************************************
 *
 *	3.1.1  Process Creation
 *
 ****************************************************************/
extern pid_t fork __PTL_NOARGS;


/****************************************************************
 *
 *	3.2.2  Terminate a Process
 *
 ****************************************************************/
extern void _exit __PTL_P((int));


/****************************************************************
 *
 *	3.4.1  Schedule Alarm
 *
 ****************************************************************/
extern unsigned int alarm __PTL_P((unsigned));


/****************************************************************
 *
 *	3.4.2  Suspend Process Execution
 *
 ****************************************************************/
extern int pause __PTL_NOARGS;


/****************************************************************
 *
 *	3.4.3  Delay Process Execution
 *
 ****************************************************************/
extern unsigned _pthread_sleep __PTL_P((unsigned int, 
					const char*, const char*, int));
extern unsigned (sleep) __PTL_P((unsigned int));
extern int usleep __PTL_P((unsigned int));


/****************************************************************
 *
 *	6.4.1  Read from a File
 *
 ****************************************************************/
extern ssize_t read __PTL_P((int, _pthread_any_t, size_t));

/* PTL API */
extern ssize_t read_timedwait __PTL_P((int, _pthread_any_t, size_t,
				       struct timespec*));


/****************************************************************
 *
 *	6.4.2  Write to a File
 *
 ****************************************************************/
extern ssize_t write __PTL_P((int, _pthread_const_any_t, size_t));

/* PTL API */
extern ssize_t write_timedwait __PTL_P((int, _pthread_const_any_t, size_t,
					struct timespec*));


/****************************************************************
 *
 *	6.1.1  Create an Inter-Process Channel
 *
 ****************************************************************/
extern int pipe __PTL_P((int[]));

#ifndef HAVE_TTYNAME_R
extern int ttyname_r __PTL_P((int, char*, size_t));
#endif

__PTL_END_DECLS

/* ---------------------------------------------------------------- */
#ifdef __PTL_ALREADY_RENAMED_UNISTD_H_
#include <sys/pthread-before.h>
#undef __PTL_ALREADY_RENAMED_UNISTD_H
#else
#include <sys/pthread-after.h>
#endif
/* ---------------------------------------------------------------- */

#endif /* __PTL_UNISTD_H__ */
